from balderich.models.resource import Collection
from balderich.utils.code import SUCCESS
from balderich.utils.exceptions import get_exception
from typing import Any, Dict, List

class TeamCollection(Collection):
    def get_team_list_by_page(self, page: int, size: int=10) -> Dict[str, Any]:
        """
        获取团队列表。数据缓存10分钟。
        
        Args:
            page参数为页数，size参数为每页大小。

        Returns:
            {
                teams: [{
                    id: integer,
                    name: string,
                    bio: integer,
                    date: integer,
                    user: {
                        uid: integer,
                        rating: integer,
                        username: string
                    },
                    nums: integer
                },],
                total: integer
            }
        """
        code, data = self.client._get(f'team/list/{page}/{size}/')
        
        if code != SUCCESS:
            raise get_exception(code)

        return data
    
    def get_team_info(self, tid: int) -> Dict[str, Any]:
        """
        获取团队详细信息。数据缓存10分钟。
        
        Args:
            tid参数为团队ID。

        Returns:
            {
                id: integer,
                name: string,
                bio: integer,
                date: integer,
                avatar: string,
                user: {
                    uid: integer,
                    rating: integer,
                    username: string
                },
                nums: integer
            }
        """
        code, data = self.client._get(f'team/{tid}/info/')
        
        if code != SUCCESS:
            raise get_exception(code)

        return data
    
    def get_team_notice(self) -> Dict[str, Any]:
        """
        获取团队通知信息。数据缓存10分钟。

        Returns:
            {
                notice: string
            }
        """
        code, data = self.client._get(f'team/notice/')
        
        if code != SUCCESS:
            raise get_exception(code)

        return data
    
    def put_team_clockin(self) -> Dict[str, Any]:
        """
        团队每日打卡

        Returns:
            {
                state: boolean,
                nums: integer
            }
        """
        code, data = self.client._put(f'team/clockin/')
        
        if code != SUCCESS:
            raise get_exception(code)

        return data
    
    def get_team_problem_list_by_page(self, page: int, size: int=10) -> Dict[str, Any]:
        """
        获取团队题目列表。数据缓存10分钟。

        Args:
            page参数为页数，size参数为每页大小。

        Returns:
            {
                problems: [{
                    id: integer,
                    title: string,
                    point: integer,
                    tags: [string,],
                    level: float,
                    solves: integer,
                },]
                total: integer
            }
        """
        code, data = self.client._get(f'team/problem/list/{page}/{size}/')
        
        if code != SUCCESS:
            raise get_exception(code)

        return data
    
    def get_team_problem_info(self, pid: int) -> Dict[str, Any]:
        """
        获取团队题目详细信息。数据缓存10分钟。

        Args:
            pid参数为题目ID。
            
        Returns:
            {
                id: interger,
                title: string,
                desc: string,
                point: number,
                tags: [string, ],
                hint: boolean,
                level: float,
                annex: boolean,
                docker: boolean,
                price: integer,
                likes: integer,
                date: integer,
                info: {
                    solved: integer,
                    wa: integer,
                }
                author: {
                    uid: integer,
                    name: string,
                    rating: integer
                }
            }
        """
        code, data = self.client._get(f'team/problem/{pid}/info/')
        
        if code != SUCCESS:
            raise get_exception(code)

        return data
    
    def get_team_contest_list_by_page(self, page: int, size: int=10) -> Dict[str, Any]:
        """
        获取团队比赛列表。数据缓存10分钟。

        Args:
            page参数为页数，size参数为每页大小。

        Returns:
            其中state的值从[0-2]，分别代表未开始、进行中和已结束。
            {
                contests: {
                    [
                        id: integer,
                        cover: string,
                        title: string,
                        start_date: integer,
                        ends_date: integer,
                        is_team: boolean,
                        desc: string,
                        state: integer
                    ],
                },
                total: integer
            }
        """
        code, data = self.client._get(f'team/contest/list/{page}/{size}/')
        
        if code != SUCCESS:
            raise get_exception(code)

        return data
    
    def get_team_contest_info(self, cid: int) -> Dict[str, Any]:
        """
        获取团队比赛详细信息。数据缓存10分钟。

        Args:
            cid参数为比赛ID。
            
        Returns:
            其中level分别为[0-4]代表个人公开赛、个人密码赛、团队公开赛和团队密码赛。
            {
                title: string,
                level: integer,
                type: integer,
                mode: integer,
                desc: string,
                cover: string,
                top_score: integer,
                decrease_score: integer,
                start_date: integer,
                ends_date: integer,
            }
        """
        code, data = self.client._get(f'team/contest/{cid}/info/')
        
        if code != SUCCESS:
            raise get_exception(code)

        return data
    
    def get_team_contest_rank_list_by_page(self, cid: int, page: int) -> Dict[str, Any]:
        """
        获取团队比赛榜单列表。相同的路径数据缓存60秒。

        Args:
            cid参数为比赛ID。page参数为页数。

        Returns:
            cetegory为类型列表，包含题目类型字符串，“1”->"10"分别代表WEB->实战。
            point为分数字段，键为题目PID，值为题目当前分数。
            problems为题目详细信息列表，每项元素都为一个包含三子元素的列表，分别为PID，解题数，题目名。
            top3为前3血字典，键为题目PID，值为前三血用户UID字符串列表。
            {
                category: [string, ],
                point: {
                    integer: integer,
                },
                problems: [[integer, integer, string], ],
                top3: {
                    integer: [string, ],
                },
                solves: [{
                    uid: integer,
                    username: string,
                    rating: integer
                    solved: string,
                    solved_time: string,
                    score: integer,
                }, ],
                team: boolean,
                total: integer
            }
        """
        code, data = self.client._get(f'team/contest/{cid}/rank/list/{page}/{size}/')
        
        if code != SUCCESS:
            raise get_exception(code)

        return data
    
    def get_team_user_list_by_page(self, page: int, size: int=10) -> Dict[str, Any]:
        """
        获取团队成员列表。数据缓存10分钟。
        
        Args:
            page参数为页数，size参数为每页大小。

        Returns:
            返回内容中role为用户角色，其中[0,1,2]分别代表[成员，管理员，队长]
            {
                users: [{
                    id: integer,
                    uid: integer,
                    username: string,
                    rating: integer,
                    alias: string,
                    date: integer,
                    role: integer
                },],
                total: integer
            }
        """
        code, data = self.client._get(f'team/user/list/{page}/{size}/')
        
        if code != SUCCESS:
            raise get_exception(code)

        return data
    
    def get_team_apply_list_by_page(self, page: int, size: int=10) -> Dict[str, Any]:
        """
        获取团队申请列表，访问此API需要团队管理员及以上权限。数据缓存10分钟。

        Args:
            page参数为页数，size参数为每页大小。

        Returns:
            {
                users: [{
                    id: integer,
                    uid: integer,
                    username: string,
                    rating: integer,
                    msg: string,
                    date: integer
                },],
                total: integer
            }
        """
        code, data = self.client._get(f'team/user/apply/list/{page}/{size}/')
        
        if code != SUCCESS:
            raise get_exception(code)

        return data
    
    def get_team_analysis_use(self) -> Dict[str, Any]:
        """
        获取团队使用情况，访问此API需要团队管理员及以上权限。数据缓存10分钟。

        Returns:
            {
                problem: {
                    now: integer,
                    max: integer
                },
                contest: {
                    now: integer,
                    max: integer
                },
                memory: {
                    now: float,
                    max: integer
                },
                person: {
                    now: integer,
                    max: integer
                },
                level: integer,
                state: integer
            }
        """
        code, data = self.client._get(f'team/analysis/use/')
        
        if code != SUCCESS:
            raise get_exception(code)

        return data
    
    def post_team_analysis_solves_curve(self, uids: List[int]) -> Dict[str, Any]:
        """
        获取团队解题曲线数据，访问此API需要团队管理员及以上权限。数据缓存10分钟。

        Args:
            uids: [integer, ]

        Returns:
            获取最近三月做题时间戳列表
            {
                integer: [interger, ]
            }
        """
        code, data = self.client._post(f'team/analysis/solves/curve/', data={
            'uids': uids
        })
        
        if code != SUCCESS:
            raise get_exception(code)

        return data
    
    def get_team_statistics_day(self, uids: str) -> Dict[str, Any]:
        """
        获取团队每日解题数据，访问此API需要团队管理员及以上权限。数据缓存10分钟。

        Args:
            uids: [integer, ]

        Returns:
            {
                integer: {
                    count: integer,
                    sum_score: integer,
                    team_count: integer,
                    team_sum_score: integer
                }
            }
        """
        code, data = self.client._post(f'team/statistics/day/', data={
            'uids': uids
        })
        
        if code != SUCCESS:
            raise get_exception(code)

        return data



SUCCESS = 10000
AUTH_NONE = 10001
AUTH_NOT_EXIST = 10002
AUTH_ERROR_SIGN = 10003
AUTH_TIMEOUT = 10004
AUTH_CALC_ERROR = 10005
AUTH_REQUEST_FAST = 10006
REQUEST_PARAM_INVALID = 10007

USER_NOT_EXIST = 11001
USER_CLOSE_FOLLOW = 11002
USER_IMAGE_NONE = 11003
USER_IMAGE_FORMAT_ERROR = 11004
USER_IMAGE_OPEN_ERROR = 11005
USER_MEMORY_NOT_ENOUGH = 11006
USER_IMAGE_NOT_EXIST = 11007

PROBLEM_NOT_EXIST = 12001
PROBLEM_PEMISSION_DENIED = 12002
PROBLEM_SHEET_NOT_EXIST = 12101
PROBLEM_SHEET_PEMISSION_DENIED = 12102

CONTEST_NOT_EXIST = 13001
CONTEST_PERMISSION_DENIED = 13002

TEAM_NOT_EXIST = 14001
TEAM_PERMISSION_DENIED = 14002
TEAM_NO_MEMBER = 14003
TEAM_METHOD_PERMISSION_DENIED = 14004
TEAM_PROBLEM_NOT_EXIST = 14101
TEAM_PROBELM_PERMISSION_DENIED = 14102
TEAM_CONTEST_NOT_EXIST = 14201
TEAM_CONTEST_PERMISSION_DENIED = 14202



from typing import Any, Dict
from balderich.models.resource import Collection
from balderich.utils.code import SUCCESS
from balderich.utils.exceptions import get_exception


class NotebookCollection(Collection):
    def get_notebook_info(self, nid: int) -> Dict[str, Any]:
        """
        获取文章信息。数据缓存1小时。

        Args:
            nid参数为文章ID。

        Returns:
            若文章文件为PDF，则content字段为Base64编码后的字符串；否则为文章正文内容。
            {
                id: integer,
                title: string,
                upload_date: integer,
                modify_date: integer,
                tag: [string, ],
                state: integer,
                content: string,
                author: {
                    uid: integer,
                    username: string,
                    rating: integer,
                    avatar: string
                },
                info: {
                    views: integer,
                    likes: integer
                },
                like: boolean
            }
        """
        code, data = self.client._get(f'notebook/{nid}/info/')

        if code != SUCCESS:
            raise get_exception(code)

        return data



from balderich.utils.code import *

class AuthNoneException(Exception): ...
class AuthNotExistException(Exception): ...
class AuthErrorSignException(Exception): ...
class AuthTimeoutException(Exception): ...
class AuthCalcErrorException(Exception): ...
class AuthRequestFastException(Exception): ...
class RequestPramInvaildException(Exception): ...

class UserNotExistException(Exception): ...
class UserCloseFollowException(Exception): ...
class UserImageNoneException(Exception): ...
class UserImageFormatErrorException(Exception): ...
class UserImageOpenErrorException(Exception): ...
class UserMemoryNotEnoughException(Exception): ...
class UserImageNotExistException(Exception): ...

class ProblemNotExistException(Exception): ...
class ProblemPermissionDeniedException(Exception): ...
class ProblemSheetNotExistException(Exception): ...
class ProblemSheetPermissionDeniedException(Exception): ...

class ContestNotExistException(Exception): ...
class ContestPermissionDeniedException(Exception): ...

class TeamNotExistException(Exception): ...
class TeamPermissionDeniedException(Exception): ...
class TeamNoMemberException(Exception): ...
class TeamMethodPermissionDeniedException(Exception): ...
class TeamProblemNotExistException(Exception): ...
class TeamProblemPermissionDeniedException(Exception): ...
class TeamContestNotExistException(Exception): ...
class TeamContestPermissionDeniedException(Exception): ...


def get_exception(code: int, msg: str='') -> Exception:
    if code == AUTH_NONE:
        return AuthNoneException(msg)
    elif code == AUTH_NOT_EXIST:
        return AuthNotExistException(msg)
    elif code == AUTH_NOT_EXIST:
        return AuthErrorSignException(msg)
    elif code == AUTH_ERROR_SIGN:
        return AuthErrorSignException(msg)
    elif code == AUTH_TIMEOUT:
        return AuthTimeoutException(msg)
    elif code == AUTH_CALC_ERROR:
        return AuthCalcErrorException(msg)
    elif code == AUTH_REQUEST_FAST:
        return AuthRequestFastException(msg)
    elif code == REQUEST_PARAM_INVALID:
        return RequestPramInvaildException(msg)

    elif code == USER_NOT_EXIST:
        return UserNotExistException(msg)
    elif code == USER_CLOSE_FOLLOW:
        return UserCloseFollowException(msg)
    elif code == USER_IMAGE_NONE:
        return UserImageNoneException(msg)
    elif code == USER_IMAGE_FORMAT_ERROR:
        return UserImageFormatErrorException(msg)
    elif code == USER_IMAGE_OPEN_ERROR:
        return UserImageOpenErrorException(msg)
    elif code == USER_MEMORY_NOT_ENOUGH:
        return UserMemoryNotEnoughException(msg)
    elif code == USER_IMAGE_NOT_EXIST:
        return UserImageNotExistException(msg)
    
    elif code == PROBLEM_NOT_EXIST:
        return ProblemNotExistException(msg)
    elif code == PROBLEM_PEMISSION_DENIED:
        return ProblemPermissionDeniedException(msg)
    elif code == PROBLEM_SHEET_NOT_EXIST:
        return ProblemSheetNotExistException(msg)
    elif code == PROBLEM_SHEET_PEMISSION_DENIED:
        return ProblemSheetPermissionDeniedException(msg)
    
    elif code == CONTEST_NOT_EXIST:
        return ContestNotExistException(msg)
    elif code == CONTEST_PERMISSION_DENIED:
        return ContestPermissionDeniedException(msg)
    
    elif code == TEAM_NOT_EXIST:
        return TeamNotExistException(msg)
    elif code == TEAM_PERMISSION_DENIED:
        return TeamPermissionDeniedException(msg)
    elif code == TEAM_NO_MEMBER:
        return TeamNoMemberException(msg)
    elif code == TEAM_METHOD_PERMISSION_DENIED:
        return TeamMethodPermissionDeniedException(msg)
    elif code == TEAM_PROBLEM_NOT_EXIST:
        return TeamProblemNotExistException(msg)
    elif code == TEAM_PROBELM_PERMISSION_DENIED:
        return TeamProblemPermissionDeniedException(msg)
    elif code == TEAM_CONTEST_NOT_EXIST:
        return TeamContestNotExistException(msg)
    elif code == TEAM_CONTEST_PERMISSION_DENIED:
        return TeamContestPermissionDeniedException(msg)



from typing import Any, Dict, List
from balderich.models.resource import Collection
from balderich.utils.code import SUCCESS
from balderich.utils.exceptions import get_exception


class ProblemCollection(Collection):
    def get_problem_list_by_page(self, page: int, size: int=10) -> Dict[str, Any]:
        """
        获取题目列表。数据缓存10分钟。

        Args:
            page参数为页数，size参数为每页大小。

        Returns:
            {
                problems: [{
                    id: integer,
                    title: string,
                    tag: [string, ],
                    wp: boolean,
                    point: integer,
                    level: float
                },]
                total: integer
            }
        """
        code, data = self.client._get(f'problem/list/{page}/{size}/')
        
        if code != SUCCESS:
            raise get_exception(code)

        return data

    def get_problem_info(self, pid: int) -> Dict[str, Any]:
        """
        获取题目详细信息。数据缓存10分钟。

        Args:
            pid参数为题目ID。

        Returns:
            {
                id: interger,
                title: string,
                desc: string,
                point: number,
                tags: [string, ],
                hint: boolean,
                level: float,
                annex: boolean,
                docker: boolean,
                price: integer,
                likes: integer,
                date: integer,
                info: {
                    solved: integer,
                    wa: integer,
                }
                author: {
                    uid: integer,
                    name: string,
                    rating: integer
                }
            }
        """
        code, data = self.client._get(f'problem/{pid}/info/')

        if code != SUCCESS:
            raise get_exception(code)

        return data

    def get_problem_wp_list_by_page(self, pid: int, page: int, size: int=10) -> Dict[str, Any]:
        """
        按页获取题目题解（WP）列表。数据缓存10分钟。

        Args:
            pid参数为题目ID，page参数为页数，size参数为每页大小，size最大为20。

        Returns:
            {
                wps: [{
                    id: integer,
                    author: {
                        uid: integer,
                        username: string,
                        rating: integer,
                        avatar: string,
                    },
                    etags: [[string, integer], ],
                    is_like: boolean,
                    title: string,
                    date: integer,
                    desc: string,
                    tags: [string, ],
                    cover: string,
                    views: integer,
                    likes: integer,
                    comments: integer
                },],
                total: integer
            }
        """
        code, data = self.client._get(f'problem/{pid}/wp/list/{page}/{size}/')

        if code != SUCCESS:
            raise get_exception(code)

        return data

    def get_problem_daily_problem(self, date: str) -> List[Dict[str, Any]]:
        """
        获取指定日期的每日一题题目。数据缓存约 2 小时。

        Args:
            date: 日期字符串，格式 YYYY-MM-DD。仅允许当年且不晚于当天（UTC）。

        Returns:
            题目列表，每项为:
            {
                id: integer,
                title: string,
                tag: [string, ],
                wp: boolean,
                info: {
                    solved: integer,
                    wrong: integer,
                    no: integer
                },
                level: float
            }
        """
        code, data = self.client._get(f'problem/daily/{date}/problem/')

        if code != SUCCESS:
            raise get_exception(code)

        return data

    def get_problem_sheet_list_by_page(self, page: int, size: int=10) -> Dict[str, Any]:
        """
        获取题单列表。数据缓存10分钟。

        Args:
            page参数为页数，size参数为每页大小。

        Returns:
            {
                sheets: [{
                    id: integer,
                    title: string,
                    stars: integer,
                    count: integer,
                    author: {
                        uid: integer,
                        name: string,
                        rating: integer
                    }
                },]
                total: integer
            }
        """
        code, data = self.client._get(f'problem/sheet/list/{page}/{size}/')
        
        if code != SUCCESS:
            raise get_exception(code)

        return data

    def get_problem_sheet_info(self, psid: int) -> Dict[str, Any]:
        """
        获取题单详细数据。数据缓存10分钟。
        
        Args:
            psid参数为题单ID。

        Returns:
            其中type(0-2)代表题单类型，分别代表（历史比赛、官方精选、用户分享）。
            {
                id: integer,
                title: string,
                stars: integer,
                count: integer,
                type: integer,
                content: string,
                author: {
                    uid: integer,
                    name: string,
                    rating: integer
                }
            }
        """
        code, data = self.client._get(f'problem/sheet/{psid}/info/')
        
        if code != SUCCESS:
            raise get_exception(code)

        return data

    def get_problem_sheet_problem_list_by_page(self, psid: int, page: int, size: int=10) -> Dict[str, Any]:
        """
        获取题单题目列表。数据缓存10分钟。

        Args:
            psid参数为题单ID。

        Returns:
            其中type(0-2)代表题单类型，分别代表（历史比赛、官方精选、用户分享）。
            {
                problems: [{
                    id: integer,
                    title: string,
                    point: integer,
                    solved: integer,
                    level: float,
                    tags: [string,],
                    index: integer
                },],
                total: integer
            }
        """
        code, data = self.client._get(f'problem/sheet/{psid}/list/{page}/{size}/')
        
        if code != SUCCESS:
            raise get_exception(code)

        return data



import io
from balderich.models.resource import Collection
from balderich.utils.code import SUCCESS
from balderich.utils.exceptions import get_exception
from typing import IO, Any, Dict, List

class UserCollection(Collection):
    def get_user_info(self, name: str) -> Dict[str, Any]:
        """
        获取用户个人信息。相同的路径数据缓存60分钟。

        Args:
            name参数可以为用户名或用户UID。
            
        Returns:
            {
                uid: integer,
                bio: string,
                intro: string,
                username: string,
                solves: integer,
                rating: integer,
                avatar: string,
                cover: string,
                register_date: integer,
                last_login_date: integer,
                email: string,
                followers: integer,
                following: integer,
                tid: integer
                team: string,
                is_vip: boolean
            }
        """
        code, data = self.client._get(f'user/{name}/info/')

        if code != SUCCESS:
            raise get_exception(code)

        return data
        
    def get_user_statistics_active(self, uid: int) -> Dict[str, Any]:
        """
        获取用户解题活跃数据。相同的路径数据缓存60分钟。

        Args:
            uid参数为用户UID。

        Returns:
            从当年1月1日至当天的每日解题数据。

            {
                start_date: integer,
                ends_date: integer,
                count: [
                    [string, integer],
                    ...
                ]
            }
        """
        code, data = self.client._get(f'user/{uid}/statistics/active/')

        if code != SUCCESS:
            raise get_exception(code)

        return data
    
    def get_user_statistics_solves(self, uid: int) -> List[Any]:
        """
        获取用户解题曲线。相同的路径数据缓存60分钟。

        Args:
            uid参数为用户UID。

        Returns:
            分类返回用户解题时间时间戳数据。

            [
                {
                    type: 0,
                    name: "WEB",
                    data: [integer, ...]
                }, {
                    type: 1,
                    name: "PWN",
                    data: [integer, ...]
                }, {
                    type: 2,
                    name: "REVERSE",
                    data: [integer, ...]
                }, {
                    type: 3,
                    name: "CRYPTO",
                    data: [integer, ...]
                }, {
                    type: 4,
                    name: "MISC",
                    data: [integer, ...]
                }, {
                    type: 5,
                    name: "MOBILE",
                    data: [integer, ...]
                }, {
                    type: 6,
                    name: "ETH",
                    data: [integer, ...]
                }, {
                    type: 7,
                    name: "IOT",
                    data: [integer, ...]
                }, {
                    type: 8,
                    name: "AI",
                    data: [integer, ...]
                }, {
                    type: 9,
                    name: "实战",
                    data: [integer, ...]
                },
            ]
        """
        code, data = self.client._get(f'user/{uid}/statistics/solves/')

        if code != SUCCESS:
            raise get_exception(code)

        return data
    
    def get_user_statistics_rating(self, uid: int) -> Dict[str, Any]:
        """
        获取用户积分曲线。相同的路径数据缓存60分钟。

        Args:
            uid参数为用户UID。

        Returns:
            返回用户参加的每场比赛排名以及积分变动数据。

            [
                {
                    date: integer,
                    rating: integer,
                    title: string,
                    rank: integer,
                    unrated: boolean,
                    nums: integer
                }
            ]
        """
        code, data = self.client._get(f'user/{uid}/statistics/rating/')

        if code != SUCCESS:
            raise get_exception(code)

        return data
    
    def get_user_statistics_radar(self, uid: int) -> Dict[str, Any]:
        """
        获取用户能力雷达图数据。相同的路径数据缓存60分钟。

        Args:
            uid参数为用户UID。

        Returns:
            返回一个列表，包含用户各方向解题数据，其中一共六项，分别代表WEB、PWN、REVERSE、CRYPTO、MISC、OTHER方向解题数据，每项数据都为[解题数, 总题数]的列表

            [
                {
                    date: integer,
                    rating: integer,
                    title: string,
                    rank: integer,
                    unrated: boolean,
                    nums: integer
                }
            ]
        """
        code, data = self.client._get(f'user/{uid}/statistics/radar/')

        if code != SUCCESS:
            raise get_exception(code)

        return data
    
    def get_user_article_list(self, uid: int, page: int, size: int) -> Dict[str, Any]:
        """
        获取用户文章列表。相同的路径数据缓存5分钟。

        Args:
            uid参数为用户UID。page参数为页数，size参数为每页大小。

        Returns:
            返回指定页的文章数据和文章总数，文章数据按照文章ID排降序。

            {
                articles: [
                    {
                        id: integer
                        title: string,
                        date: integer,
                        type: integer
                    }, ...
                ],
                total: integer
            }
        """
        code, data = self.client._get(f'user/{uid}/article/list/{page}/{size}/')

        if code != SUCCESS:
            raise get_exception(code)

        return data
    
    def get_user_following_list(self, uid: int, page: int, size: int) -> Dict[str, Any]:
        """
        获取用户关注列表。相同的路径数据缓存10分钟。

        Args:
            uid参数为用户UID。page参数为页数，size参数为每页大小。

        Returns:
            返回指定页的关注列表数据，数据按照关注时间排降序。

            [
                {
                    uid: integer,
                    username: string,
                    bio: string,
                    avatar: string
                },
            ]
        """
        code, data = self.client._get(f'user/{uid}/following/list/{page}/{size}/')

        if code != SUCCESS:
            raise get_exception(code)

        return data
    
    def get_user_follower_list(self, uid: int, page: int, size: int) -> Dict[str, Any]:
        """
        获取用户粉丝列表。相同的路径数据缓存10分钟。

        Args:
            uid参数为用户UID。page参数为页数，size参数为每页大小。

        Returns:
            返回指定页的粉丝列表数据，数据按照关注时间排降序。

            [
                {
                    uid: integer,
                    username: string,
                    bio: string,
                    avatar: string
                },
            ]
        """
        code, data = self.client._get(f'user/{uid}/follower/list/{page}/{size}/')

        if code != SUCCESS:
            raise get_exception(code)

        return data
    
    def get_user_picturebed_used(self) -> Dict[str, Any]:
        """
        获取图床使用情况。数据缓存10分钟。

        Returns:
            返回图床使用情况。
            
            {
                used_mem: integer,
                max_mem: integer,
                total: integer,
            }
        """
        code, data = self._get(f'user/picturebed/used/')

        if code != SUCCESS:
            raise get_exception(code)

        return data
    
    def get_user_picturebed_used(self, page: int, size: int) -> Dict[str, Any]:
        """
        获取图床列表。数据缓存10分钟。
        
        Args:
            page参数为页数，size参数为每页大小。

        Returns:
            返回指定页的图床列表数据，数据按照id排降序。

            {
                pictures: [
                    {
                        id: integer,
                        name: string,
                        date: integer,
                        size: integer,
                        src: string
                    }, ...
                ],
                total: integer
            }
        """
        code, data = self.client._get(f'user/picturebed/list/{page}/{size}/')

        if code != SUCCESS:
            raise get_exception(code)

        return data
    
    def post_user_picturebed_upload(self, filename: str, image: IO) -> Dict[str, Any]:
        """
        图床上传图片

        Args:
            filename: 文件名
            image: 文件IO流

        Returns:
            {
                id: integer,
                name: string,
                date: integer,
                size: integer,
                url: string
            }
        """
        code, data = self.client._post(f'user/picturebed/upload/', files={
            'image': (filename, image.read())
        })
        if code != SUCCESS:
            raise get_exception(code)

        return data
    
    def post_user_picturebed_download(self, pid: int) -> io.BytesIO:
        """
        图床下载图片

        Args:
            pid参数为图片ID。

        Returns:
            bytes IO流
        """
        res = self.client._post(f'user/picturebed/{pid}/download/', parse=False)

        if res.headers['Content-Type'] != 'application/octet-stream':
            raise get_exception(res.json()['code'])

        return io.BytesIO(res.content)



from typing import Any, Dict
from balderich.models.resource import Collection
from balderich.utils.code import SUCCESS
from balderich.utils.exceptions import get_exception


class ContestCollection(Collection):
    def get_contest_list(self, type: int, page: int) -> Dict[str, Any]:
        """
        获取比赛列表。数据缓存10分钟。

        Args:
            type: 比赛类型
                0 -> 公开赛事
                1 -> 私密赛事
            page: 页面参数

        Returns:
            {
                contests: [{
                    id: integer,
                    cover: string,
                    title: string,
                    level: integer,
                    mode: integer,
                    start_date: integer,
                    ends_date: integer,
                    desc: integer,
                    state: integer,
                    count: integer
                },],
                total: integer
            }
        """
        code, data = self.client._get(f'contest/{type}/list/{page}/')
        if code != SUCCESS:
            raise get_exception(code)

        return data
    
    def get_contest_info(self, cid: int) -> Dict[str, Any]:
        """
        获取比赛详细信息。数据缓存10分钟。

        Args:
            cid: 比赛ID
        
        Returns:
            {
                id: integer,
                title: string,
                level: integer,
                type: integer,
                mode: integer,
                desc: integer,
                cover: string,
                top_score: integer,
                descrease_score: integer,
                start_date: integer,
                ends_date: integer,
                is_team: boolean
            }
        """
        code, data = self.client._get(f'contest/{cid}/info/')
        if code != SUCCESS:
            raise get_exception(code)

        return data
    

    def get_contest_rank_list(self, cid: int, page: int) -> Dict[str, Any]:
        """
        获取比赛榜单信息。相同路径数据缓存60秒。

        Args:
            cid: 比赛ID
        
        Returns:
            cetegory为类型列表，包含题目类型字符串，“1”->"10"分别代表WEB->实战。
            point为分数字段，键为题目PID，值为题目当前分数。
            problems为题目详细信息列表，每项元素都为一个包含三子元素的列表，分别为PID，解题数，题目名。
            top3为前3血字典，键为题目PID，值为前三血用户UID字符串列表。
            {
                category: [string, ],
                point: {
                    integer: integer,
                },
                problems: [[integer, integer, string], ],
                top3: {
                    integer: [string, ],
                },
                solves: [{
                    uid: integer,
                    username: string,
                    rating: integer
                    solved: string,
                    solved_time: string,
                    score: integer,
                }, ],
                team: boolean,
                total: integer
            }
        """
        code, data = self.client._get(f'contest/{cid}/rank/list/{page}/')
        if code != SUCCESS:
            raise get_exception(code)

        return data


import time
import json
import hashlib
import requests
from functools import cached_property
from typing import Tuple, Union, IO
from balderich.models.user import UserCollection
from balderich.models.contest import ContestCollection
from balderich.models.problem import ProblemCollection
from balderich.models.team import TeamCollection
from balderich.models.notebook import NotebookCollection

class AuthConfig:
    """
    Client authorized configure setting

    Example:

        >>> config = AuthConfig(key, secret)
    
        or

        >>> config = AuthConfig.load_config_file('config.json')
        >>> config = AuthConfig.load_config_file(open('config.json', 'rb'))
    """
    def __init__(self, key: str, secret: str) -> None:
        self.key = key
        self.secret = secret
    
    @staticmethod
    def load_config_file(filepath: Union[str, IO]):
        if isinstance(filepath, str):
            file = open(filepath, 'rb')
        else:
            file = filepath
        
        data = json.load(file)

        self = AuthConfig(data['key'], data['secret'])
        return self

    def sign(self, path: str, timestamp: int = None, prefix: str='/v2/api/') -> Tuple[str, int]:
        if timestamp is None:
            timestamp = int(time.time())
        
        res = hashlib.sha256(f'{prefix}{path}#{self.key}#{timestamp}#{self.secret}'.encode()).hexdigest()

        return res, timestamp


class NSSClient:
    """
    A client for communication with Balderich

    Example:
        
        >>> import balderich

        then 
        
        >>> client = balerich.NSSClient(key='****', secret='*****')

        or

        >>> client = balderich.NSSClient(balderich.AuthConfig.load_config_file('key.json'))
    """
    def __init__(self,
        auth_cofig: AuthConfig=None,
        key: str=None,
        secret: str=None,
        url: str=None
    ) -> None:
        if url is None:
            url = 'https://www.nssctf.cn/v2/api/'
        if auth_cofig is None:
            auth_cofig = AuthConfig(key, secret)
        
        self.url = url
        self.auth_config = auth_cofig

    @cached_property
    def user(self):
        return UserCollection(client=self)

    @cached_property
    def problem(self):
        return ProblemCollection(client=self)

    @cached_property
    def notebook(self):
        return NotebookCollection(client=self)

    @cached_property
    def contest(self):
        return ContestCollection(client=self)

    @cached_property
    def team(self):
        return TeamCollection(client=self)
    
    @property
    def key(self):
        return self.auth_config.key

    def _get(self, path: str) -> Tuple[int, Union[int, str, float, bool, list, dict]]:
        sign, timestamp = self.auth_config.sign(path)
        res = requests.get(self.url+path, params={
            'key': self.key,
            'time': timestamp,
            'sign': sign
        })

        res = res.json()
        code = res['code']
        data = res['data']

        return code, data

    def _post(self, path: str, data: dict=None, files: dict=None, parse: bool=True) -> Union[requests.Response, Tuple[int, Union[int, str, float, bool, list, dict]]]:
        if data is None:
            dict = {}
        
        if files is None:
            files = files or {}
        
        sign, timestamp = self.auth_config.sign(path)
        res = requests.post(self.url+path, params={
            'key': self.key,
            'time': timestamp,
            'sign': sign
        }, data=data, files=files)
        print(res)
        if not parse:
            return res
        
        res = res.json()

        code = res['code']
        data = res['data']

        return code, data
    
    def _put(self, path: str, data: dict=None) -> Tuple[int, Union[int, str, float, bool, list, dict]]:
        if data is None:
            dict = {}

        sign, timestamp = self.auth_config.sign(path)
        res = requests.put(self.url+path, params={
            'key': self.key,
            'time': timestamp,
            'sign': sign
        }, data=data).json()

        code = res['code']
        data = res['data']

        return code, data



from os import path as os_path
from setuptools import setup

this_directory = os_path.abspath(os_path.dirname(__file__))

def read_file(filename):
    with open(os_path.join(this_directory, filename), encoding='utf-8') as f:
        long_description = f.read()
    return long_description

setup(
    long_description=read_file('README.md')
)


from __future__ import annotations

class Collection:
    """
    A base class fro representing all objects of a particular type on the server.
    """

    def __init__(self, client = None):
        self.client = client
